@@ -52,7 +52,7 @@ for table in BUILT_IN_CHAINS.keys():
 class ebtables(object):
     ipv = "eb"
     name = "ebtables"
-    zones_supported = False # ebtables only supported with direct interface
+    policies_supported = False # ebtables only supported with direct interface
 
     def __init__(self):
         self._command = COMMANDS[self.ipv]
@@ -201,7 +201,6 @@ class ebtables(object):
         if status != 0:
             raise ValueError("'%s %s' failed: %s" % (self._restore_command,
                                                      " ".join(args), ret))
-        return ret
 
     def set_rule(self, rule, log_denied):
         self._rule_validate(rule)
