@@ -42,6 +42,7 @@ from firewall.core.fw_policies import FirewallPolicies
 from firewall.core.fw_ipset import FirewallIPSet
 from firewall.core.fw_transaction import FirewallTransaction
 from firewall.core.fw_helper import FirewallHelper
+from firewall.core.fw_policy import FirewallPolicy
 from firewall.core.fw_nm import nm_get_bus_name, nm_get_interfaces_in_zone
 from firewall.core.logger import log
 from firewall.core.io.firewalld_conf import firewalld_conf
@@ -52,6 +53,7 @@ from firewall.core.io.zone import zone_reader, Zone
 from firewall.core.io.ipset import ipset_reader
 from firewall.core.ipset import IPSET_TYPES
 from firewall.core.io.helper import helper_reader
+from firewall.core.io.policy import policy_reader
 from firewall import errors
 from firewall.errors import FirewallError
 
@@ -76,10 +78,10 @@ class Firewall(object):
         else:
             self.ip4tables_backend = ipXtables.ip4tables(self)
             self.ip4tables_enabled = True
-            self.ip4tables_supported_icmp_types = [ ]
+            self.ipv4_supported_icmp_types = [ ]
             self.ip6tables_backend = ipXtables.ip6tables(self)
             self.ip6tables_enabled = True
-            self.ip6tables_supported_icmp_types = [ ]
+            self.ipv6_supported_icmp_types = [ ]
             self.ebtables_backend = ebtables.ebtables()
             self.ebtables_enabled = True
             self.ipset_backend = ipset.ipset()
@@ -98,6 +100,7 @@ class Firewall(object):
         self.policies = FirewallPolicies()
         self.ipset = FirewallIPSet(self)
         self.helper = FirewallHelper(self)
+        self.policy = FirewallPolicy(self)
 
         self.__init_vars()
 
@@ -123,9 +126,7 @@ class Firewall(object):
         self._firewall_backend = config.FALLBACK_FIREWALL_BACKEND
         self._flush_all_on_reload = config.FALLBACK_FLUSH_ALL_ON_RELOAD
         self._rfc3964_ipv4 = config.FALLBACK_RFC3964_IPV4
-
-    def individual_calls(self):
-        return self._individual_calls
+        self._allow_zone_drifting = config.FALLBACK_ALLOW_ZONE_DRIFTING
 
     def _check_tables(self):
         # check if iptables, ip6tables and ebtables are usable, else disable
@@ -171,11 +172,13 @@ class Firewall(object):
                 log.warning("iptables-restore and iptables are missing, "
                             "disabling IPv4 firewall.")
                 self.ip4tables_enabled = False
-        if self.ip4tables_enabled:
-            self.ip4tables_supported_icmp_types = \
-                self.ip4tables_backend.supported_icmp_types()
+        if self.nftables_enabled:
+            self.ipv4_supported_icmp_types = self.nftables_backend.supported_icmp_types("ipv4")
         else:
-            self.ip4tables_supported_icmp_types = [ ]
+            if self.ip4tables_enabled:
+                self.ipv4_supported_icmp_types = self.ip4tables_backend.supported_icmp_types()
+            else:
+                self.ipv4_supported_icmp_types = [ ]
         self.ip6tables_backend.fill_exists()
         if not self.ip6tables_backend.restore_command_exists:
             if self.ip6tables_backend.command_exists:
@@ -185,11 +188,13 @@ class Firewall(object):
                 log.warning("ip6tables-restore and ip6tables are missing, "
                             "disabling IPv6 firewall.")
                 self.ip6tables_enabled = False
-        if self.ip6tables_enabled:
-            self.ip6tables_supported_icmp_types = \
-                self.ip6tables_backend.supported_icmp_types()
+        if self.nftables_enabled:
+            self.ipv6_supported_icmp_types = self.nftables_backend.supported_icmp_types("ipv6")
         else:
-            self.ip6tables_supported_icmp_types = [ ]
+            if self.ip6tables_enabled:
+                self.ipv6_supported_icmp_types = self.ip6tables_backend.supported_icmp_types()
+            else:
+                self.ipv6_supported_icmp_types = [ ]
         self.ebtables_backend.fill_exists()
         if not self.ebtables_backend.restore_command_exists:
             if self.ebtables_backend.command_exists:
@@ -286,6 +291,20 @@ class Firewall(object):
                 log.debug1("RFC3964_IPv4 is set to '%s'",
                            self._rfc3964_ipv4)
 
+            if self._firewalld_conf.get("AllowZoneDrifting"):
+                value = self._firewalld_conf.get("AllowZoneDrifting")
+                if value.lower() in [ "no", "false" ]:
+                    self._allow_zone_drifting = False
+                else:
+                    self._allow_zone_drifting = True
+                    if not self._offline:
+                        log.warning("AllowZoneDrifting is enabled. This is considered "
+                                    "an insecure configuration option. It will be "
+                                    "removed in a future release. Please consider "
+                                    "disabling it now.")
+                log.debug1("AllowZoneDrifting is set to '%s'",
+                           self._allow_zone_drifting)
+
         self.config.set_firewalld_conf(copy.deepcopy(self._firewalld_conf))
 
         self._select_firewall_backend(self._firewall_backend)
@@ -338,6 +357,10 @@ class Firewall(object):
             log.fatal("No zones found.")
             sys.exit(1)
 
+        # load policy files
+        self._loader(config.FIREWALLD_POLICIES, "policy")
+        self._loader(config.ETC_FIREWALLD_POLICIES, "policy")
+
         # check minimum required zones
         error = False
         for z in [ "block", "drop", "trusted" ]:
@@ -431,6 +454,10 @@ class Firewall(object):
         self.zone.change_default_zone(None, self._default_zone,
                                       use_transaction=transaction)
 
+        # apply policies
+        log.debug1("Applying used policies")
+        self.policy.apply_policies(use_transaction=transaction)
+
         # Execute transaction
         transaction.execute(True)
 
@@ -591,6 +618,19 @@ class Firewall(object):
                     self.helper.add_helper(obj)
                     # add a deep copy to the configuration interface
                     self.config.add_helper(copy.deepcopy(obj))
+                elif reader_type == "policy":
+                    obj = policy_reader(filename, path)
+                    if obj.name in self.policy.get_policies():
+                        orig_obj = self.policy.get_policy(obj.name)
+                        log.debug1("  Overloads %s '%s' ('%s/%s')", reader_type,
+                                   orig_obj.name, orig_obj.path,
+                                   orig_obj.filename)
+                        self.policy.remove_policy(orig_obj.name)
+                    elif obj.path.startswith(config.ETC_FIREWALLD):
+                        obj.default = True
+                    self.policy.add_policy(obj)
+                    # add a deep copy to the configuration interface
+                    self.config.add_policy_object(copy.deepcopy(obj))
                 else:
                     log.fatal("Unknown reader type %s", reader_type)
             except FirewallError as msg:
@@ -622,12 +662,14 @@ class Firewall(object):
         self.config.cleanup()
         self.direct.cleanup()
         self.policies.cleanup()
+        self.policy.cleanup()
         self._firewalld_conf.cleanup()
         self.__init_vars()
 
     def stop(self):
         if self.cleanup_on_exit and not self._offline:
             self.flush()
+            self.ipset.flush()
             self.set_policy("ACCEPT")
             self.modules_backend.unload_firewall_modules()
 
@@ -863,6 +905,12 @@ class Firewall(object):
         if self._panic:
             raise FirewallError(errors.PANIC_MODE)
 
+    def check_policy(self, policy):
+        _policy = policy
+        if _policy not in self.policy.get_policies():
+            raise FirewallError(errors.INVALID_POLICY, _policy)
+        return _policy
+
     def check_zone(self, zone):
         _zone = zone
         if not _zone or _zone == "":
@@ -932,6 +980,10 @@ class Firewall(object):
             _direct_config = self.direct.get_runtime_config()
             _old_dz = self.get_default_zone()
 
+        _ipset_objs = []
+        for _name in self.ipset.get_ipsets():
+            _ipset_objs.append(self.ipset.get_ipset(_name))
+
         if not _panic:
             self.set_policy("DROP")
 
@@ -946,6 +998,16 @@ class Firewall(object):
             # etc. We'll re-raise it at the end.
             start_exception = e
 
+        # destroy ipsets no longer in the permanent configuration
+        if flush_all:
+            for obj in _ipset_objs:
+                if not self.ipset.query_ipset(obj.name):
+                    for backend in self.ipset.backends():
+                        # nftables sets are part of the normal firewall ruleset.
+                        if backend.name == "nftables":
+                            continue
+                        backend.set_destroy(obj.name)
+
         if not flush_all:
             # handle interfaces in the default zone and move them to the new
             # default zone if it changed
@@ -969,8 +1031,11 @@ class Firewall(object):
             # add interfaces to zones again
             for zone in self.zone.get_zones():
                 if zone in _zone_interfaces:
-                    self.zone.set_settings(zone, { "interfaces":
-                                                   _zone_interfaces[zone] })
+
+                    for interface_id in _zone_interfaces[zone]:
+                        self.zone.change_zone_of_interface(zone, interface_id,
+                                                           _zone_interfaces[zone][interface_id]["sender"])
+
                     del _zone_interfaces[zone]
                 else:
                     log.info1("New zone '%s'.", zone)
@@ -980,6 +1045,19 @@ class Firewall(object):
                     del _zone_interfaces[zone]
             del _zone_interfaces
 
+            # restore runtime-only ipsets
+            for obj in _ipset_objs:
+                if self.ipset.query_ipset(obj.name):
+                    for entry in obj.entries:
+                        try:
+                            self.ipset.add_entry(obj.name, entry)
+                        except FirewallError as msg:
+                            if msg.code != errors.ALREADY_ENABLED:
+                                raise msg
+                else:
+                    self.ipset.add_ipset(obj)
+                    self.ipset.apply_ipset(obj.name)
+
             # restore direct config
             self.direct.set_config(_direct_config)
 
@@ -988,7 +1066,7 @@ class Firewall(object):
         if nm_bus_name:
             for zone in self.zone.get_zones() + [""]:
                 for interface in nm_get_interfaces_in_zone(zone):
-                    self.zone.add_interface(zone, interface, sender=nm_bus_name)
+                    self.zone.change_zone_of_interface(zone, interface, sender=nm_bus_name)
 
         self._panic = _panic
         if not self._panic:
@@ -1075,3 +1153,38 @@ class Firewall(object):
                     self.zone.change_zone_of_interface("", iface)
         else:
             raise FirewallError(errors.ZONE_ALREADY_SET, _zone)
+
+    def combine_runtime_with_permanent_settings(self, permanent, runtime):
+        combined = permanent.copy()
+
+        for key,value in runtime.items():
+            # omit empty entries
+            if value or isinstance(value, bool):
+                combined[key] = value
+            # make sure to remove values that were in permanent, but no
+            # longer in runtime.
+            elif key in combined:
+                del combined[key]
+
+        return combined
+
+    def get_added_and_removed_settings(self, old_settings, new_settings):
+        add_settings = {}
+        remove_settings = {}
+        for key in (set(old_settings.keys()) | set(new_settings.keys())):
+            if key in new_settings:
+                if isinstance(new_settings[key], list):
+                    old = set(old_settings[key] if key in old_settings else [])
+                    add_settings[key] = list(set(new_settings[key]) - old)
+                    remove_settings[key] = list((old ^ set(new_settings[key])) & old)
+                # check for bool or int because dbus.Boolean is a subclass of
+                # int (because bool can't be subclassed).
+                elif isinstance(new_settings[key], bool) or isinstance(new_settings[key], int):
+                    if not old_settings[key] and new_settings[key]:
+                        add_settings[key] = True
+                    elif old_settings[key] and not new_settings[key]:
+                        remove_settings[key] = False
+                else:
+                    raise FirewallError(errors.INVALID_SETTING, "Unhandled setting type {} key {}".format(type(new_settings[key]), key))
+
+        return (add_settings, remove_settings)
