@@ -27,7 +27,8 @@ __all__ = [ "PY2", "getPortID", "getPortRange", "portStr", "getServiceName",
             "check_single_address", "check_mac", "uniqify", "ppid_of_pid",
             "max_zone_name_len", "checkUser", "checkUid", "checkCommand",
             "checkContext", "joinArgs", "splitArgs",
-            "b2u", "u2b", "u2b_if_py2", "max_policy_name_len"]
+            "b2u", "u2b", "u2b_if_py2", "max_policy_name_len",
+            "stripNonPrintableCharacters"]
 
 import socket
 import os
@@ -42,6 +43,15 @@ from firewall.config import FIREWALLD_TEMPDIR, FIREWALLD_PIDFILE
 
 PY2 = sys.version < '3'
 
+NOPRINT_TRANS_TABLE = {
+    # Limit to C0 and C1 code points. Building entries for all unicode code
+    # points requires too much memory.
+    # C0 = [0, 31]
+    # C1 = [127, 159]
+    #
+    i: None for i in range(0, 160) if not (i > 31 and i < 127)
+}
+
 def getPortID(port):
     """ Check and Get port id from port string or port id using socket.getservbyname
 
@@ -311,6 +321,9 @@ def checkIPnMask(ip):
                 return False
     return True
 
+def stripNonPrintableCharacters(rule_str):
+    return rule_str.translate(NOPRINT_TRANS_TABLE)
+
 def checkIP6nMask(ip):
     if "/" in ip:
         addr = ip[:ip.index("/")]
